#include <PalmOS.h>
#include <VFSMgr.h>

#include "MyInstall.h"
#include "MyInstall_Res.h"

#define MAX_FILENAME 1024


#ifdef DRIVE_MODE
static UInt16 dmLibRefNum;
static Boolean dmOn = false;
static Boolean sony = false;

typedef struct {
    UInt16 media;
    UInt16 unknown;
    UInt16 state;
} SCSIStatus;


Err DMLOpen( UInt16 refNum )
     SYS_TRAP( sysLibTrapOpen );

Err DMLClose( UInt16 refNum )
     SYS_TRAP( sysLibTrapClose );

Err DMEnable( UInt16 refNum, Boolean trace )
     SYS_TRAP( 0x0805 );

Err DMDisable( UInt16 refNum )
     SYS_TRAP( 0x0806 );

UInt32 DMState( UInt16 refNum )
     SYS_TRAP( 0x0807 );

Err SCSIOpen( UInt16 refNum, UInt16 volRefNum )
     SYS_TRAP( sysLibTrapOpen );

Err SCSIClose( UInt16 refNum )
     SYS_TRAP( sysLibTrapClose );
     
Err SCSIGetStatus( UInt16 refNum, SCSIStatus* sP )
     SYS_TRAP( sysLibTrapCustom + 1 );

static Boolean DriveMode( Boolean state, UInt32 cookie )
{
    if ( !! state == !! dmOn )
        return true;

    if ( state ) {
        if ( errNone == SysLibLoad( sysFileTLibrary, 'PDM2', &dmLibRefNum ) ) {
            Message( "Loaded palmOne library.", cookie );
            sony = false;
            if ( errNone == DMLOpen( dmLibRefNum ) ) {
                if ( errNone == DMEnable( dmLibRefNum, false ) ) {
                    dmOn = true;
                    return true;
                }
                DMLClose( dmLibRefNum );
            }
            SysLibRemove( dmLibRefNum );
            return false;
        }
        else {
            if ( errNone == SysLibLoad( sysFileTLibrary, 'SlSc', &dmLibRefNum ) ) {
                Message( "Loaded Sony library.", cookie );
                sony = true;
                if ( errNone == SCSIOpen( dmLibRefNum, 1 ) ) {
                    dmOn = true;
                    return true;
                }
                SysLibRemove( dmLibRefNum );
            }
            return false;
        }
    }
    else {
        if ( sony ) {
            SCSIStatus s;
            do {
                 SCSIGetStatus( dmLibRefNum, &s );
                 if ( s.state & 4 )
                     FrmAlert( altBusyWriting );
            } while ( s.state & 4 );
            SCSIClose( dmLibRefNum );
        }
        else {
            DMDisable( dmLibRefNum );
            DMLClose( dmLibRefNum );
        }
        SysLibRemove( dmLibRefNum );
        dmOn = false;
        return true;
    }
}
#endif


static void CopyFromDir( UInt16 volRefNum, Char* dir, Messager Message, UInt32 cookie, Boolean* copiedP, Boolean* missingP )
{
    UInt32        iterator;
    FileInfoType  info;
    FileRef       dirRef;
    Err           err;
    Char          nameBuf[ MAX_FILENAME ];

    err = VFSFileOpen( volRefNum, dir, vfsModeRead, &dirRef );

    if ( err != errNone )
        return;

    info.nameP      = nameBuf;
    info.nameBufLen = MAX_FILENAME;

    iterator = vfsIteratorStart;

    while ( iterator != vfsIteratorStop  ) {
        FileRef  fileRef;
        UInt16   attributes;
        UInt32   creator;
        UInt32   type;
        Char     dbName[ dmDBNameLength ];
        Char     fileName[ MAX_FILENAME ];
        LocalID  id;
        UInt16   card;
        Int16    i;

        err = VFSDirEntryEnumerate( dirRef, &iterator, &info );

        if ( err != errNone )
            break;

        if ( info.attributes & vfsFileAttrDirectory )
            continue;

        if ( MAX_FILENAME <= StrLen( info.nameP ) + StrLen( dir ) + 2 )
            continue;

        if ( StrLen( info.nameP ) < 3 ||
             ( StrCaselessCompare( info.nameP + StrLen( info.nameP ) - 3, "prc" )
             && StrCaselessCompare( info.nameP + StrLen( info.nameP ) - 3, "pdb" ) ) )
            continue;

        StrCopy( fileName, dir );
        StrCopy( fileName + StrLen( fileName ), "/" );
        StrCopy( fileName + StrLen( fileName ), info.nameP );

        err = VFSFileOpen( volRefNum, fileName, vfsModeRead, &fileRef );

        if ( err != errNone ) {
            if ( missingP != NULL )
                *missingP = true;
            continue;
        }

        err = VFSFileDBInfo( fileRef, dbName, &attributes,
                  NULL, NULL, NULL,  NULL, NULL, NULL, NULL,
                  &type, &creator, NULL );

        if ( err != errNone ) {
            Message("(no name)...error", cookie );
            if ( missingP != NULL )
                *missingP = true;
            continue;
        }

        VFSFileClose( fileRef );

        Message( dbName, cookie );

        for ( i = MemNumCards() - 1 ; 0 <= i ; i-- ) {
             id = DmFindDatabase( i, dbName );
             if ( id != NULL ) {

                 if ( errNone != DmDeleteDatabase( i, id ) ) {
                     SysNotifyDBInfoType d;
                     Boolean deleted = false;

                     if ( errNone == DmDatabaseInfo( i, id, d.dbName, &( d.attributes ), NULL, NULL,
                               NULL, NULL, NULL, NULL, NULL, &d.type, &d.creator ) ) {
                         SysNotifyParamType  n;

                         MemSet( &n, sizeof( n ), 0 );
                         n.notifyType  = sysNotifyDeleteProtectedEvent;
                         n.broadcaster = notifyCreator;
                         d.dbID = id;
                         d.cardNo = i;
                         n.notifyDetailsP = &d;

                         SysNotifyBroadcast( &n );

                         if ( errNone == DmDeleteDatabase( i, id ) ) {
                             deleted = true;;
                         }
                     }
                     if ( ! deleted ) {
                         if ( missingP != NULL )
                             *missingP = true;
                         continue;
                     }
                 }
             }
        }

        err = VFSImportDatabaseFromFile(
                  volRefNum,
                  fileName, &card, &id );

        if ( err == errNone ) {
            UInt32 crDate;
            UInt32 modDate;
            UInt16 attr;

            err = DmDatabaseInfo( card, id, NULL, &attr, NULL, &crDate,
                      &modDate, NULL, NULL, NULL, NULL, NULL, NULL );

            if ( err == errNone ) {
                if ( ! ( attr & dmHdrAttrBackup ) ) {
                    attr |= dmHdrAttrBackup;

                    DmSetDatabaseInfo( card, id, NULL, &attr, NULL, NULL,
                        NULL, NULL, NULL, NULL, NULL, NULL, NULL );
                }

                if ( crDate == 0 ) {
                    crDate = TimGetSeconds();

                    DmSetDatabaseInfo( card, id, NULL, NULL, NULL, &crDate,
                        NULL, NULL, NULL, NULL, NULL, NULL, NULL );
                }
                if ( modDate == 0 ) {
                    modDate = TimGetSeconds();

                    DmSetDatabaseInfo( card, id, NULL, NULL, NULL, NULL,
                        &modDate, NULL, NULL, NULL, NULL, NULL, NULL );
                }
            }
            else {
                DmDeleteDatabase( card, id );
            }
        }

        if ( err != errNone ) {
            Message("...error", cookie);
            if ( missingP != NULL )
                *missingP = true;
            continue;
        }
        else {
            if ( copiedP != NULL )
                *copiedP = true;
        }

        VFSFileDelete( volRefNum, fileName );
    }
}



void CopyFiles( UInt16 vol, Messager Message, UInt32 cookie, Boolean* copiedP, Boolean* missingP )
{
    UInt16 volRefNum;
    UInt32 volIterator;
    Err    err;

    if ( missingP != NULL )
        *missingP = false;

    if ( copiedP != NULL )
        *copiedP = false;

    for ( volIterator = vfsIteratorStart ; volIterator != vfsIteratorStop ; ) {
        err = VFSVolumeEnumerate( &volRefNum, &volIterator );
        if ( err == errNone && ( vol == ALL_VOLUMES || vol == volRefNum ) ) {
            Char driveMsg[ 40 ];

            StrPrintF( driveMsg, "Scanning drive %d...", volRefNum );
            Message( driveMsg, cookie );
            CopyFromDir( volRefNum, "/MyInstall", Message, cookie, copiedP, missingP );
        }
    }
}



